#include <LiquidCrystal_I2C.h>
#include "I2Cdev.h"
#include "MPU6050_6Axis_MotionApps20.h"
#if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
#include "Wire.h"
#endif

/*--------------------
 * I2C LCD
 --------------------*/

LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);
const int stateOn = 1;
const int stateOff = 2;
const int stateConnect = 3;


/*--------------------
 * Pin const
 *--------------------*/

// Toggle button
const int inPin = 8;        // the number of the input pin
const int outPin = 12;      // the number of the output pin
const int ledPin = 5;       // LED connected to digital pin 5

// 7 segment
const int latchPin = 7;     //to IC pin 12
const int dataPin = 3;      //to IC pin 14
const int clockPin = 4;     //to IC pin 11
const int seismicPin = A0;

/*--------------------
 * Global variable
 * --------------------*/
//
int stateLED = 1;

int valGlobal = 0;

// Toggle button
int state = HIGH;           // the current state of the output pin
int reading;                // the current reading from the input pin
int previous = LOW;         // the previous reading from the input pin
long time = 0;              // the last time the output pin was toggled
long debounce = 200;        // the debounce time, increase if the output flickers


// Seven Segment
const byte digit[] = {64, 121, 36, 48, 25, 18, 2, 120, 0, 16};
byte magnitude;
int lastSeismoDetected;
int loopNumber;

// Gyro Sensor
MPU6050 mpu;
#define INTERRUPT_PIN 2       // use pin 2 on Arduino Uno
#define TIME_WAVE_STORE 100   //
// MPU control/status vars
bool dmpReady = false;  // set true if DMP init was successful
uint8_t mpuIntStatus;   // holds actual interrupt status byte from MPU
uint8_t devStatus;      // return status after each device operation (0 = success, !0 = error)
uint16_t packetSize;    // expected DMP packet size (default is 42 bytes)
uint16_t fifoCount;     // count of all bytes currently in FIFO
uint8_t fifoBuffer[64]; // FIFO storage buffer
// orientation/motion vars
Quaternion q;           // [w, x, y, z]         quaternion container
VectorInt16 aa;         // [x, y, z]            accel sensor measurements
VectorInt16 aaReal;     // [x, y, z]            gravity-free accel sensor measurements
VectorInt16 aaWorld;    // [x, y, z]            world-frame accel sensor measurements
VectorInt16 aaWorldPrev[TIME_WAVE_STORE];
VectorFloat gravity;    // [x, y, z]            gravity vector
float euler[3];         // [psi, theta, phi]    Euler angle container
float ypr[3];           // [yaw, pitch, roll]   yaw/pitch/roll container and gravity vector
bool isChangeStore[TIME_WAVE_STORE];  // Store is has a change within TIME_WAVE_STORE time before
double amplitude;
int magnitudeC;
//INTERRUPT DETECTION ROUTINE
volatile bool mpuInterrupt = false; // indicates whether MPU interrupt pin has gone high

/*--------------------
 * Declaration
 *--------------------*/

void dmpDataReady();
void toggleButtonInit();
void sevenSegmentInit();
void lcdInit();
void gyroInit();
void changeShift();
bool isChange();
double magnitudeConvert();
void displaySevenSegment(int);
void ledActive();
void ledDeactive();
void gyroOn();
void gyroOff();
void printLCD(int,int,String);
void clearLCD();
bool isState(int);

void setup() {
  
  Serial.begin(115200);
  
  // Toggle button
  toggleButtonInit();
  
  // Seven Segment
  sevenSegmentInit();

  // LCD
  lcdInit();
  
  // Gyro Sensor
  gyroInit();

}

int iter = 0;
bool detected = 0;
void loop() {
 
  reading = digitalRead(inPin);
  if (reading == HIGH && previous == LOW && millis() - time > debounce) {
    if (state == HIGH)
      state = LOW;
    else
      state = HIGH;
    time = millis();    
  }
  digitalWrite(outPin, state);
  previous = reading;

  if(state == HIGH){
    if(!isState(stateOn)){
      clearLCD();
      printLCD(0,0,"Arduino ON");
      delay(2000);
      lcd.noBacklight();
    }
    gyroOn();
  } else {
    gyroOff();
    if(!isState(stateOff)){
      clearLCD();
      printLCD(0,0,"Arduino OFF");
    }
  }
}


/*--------------------
 * Implementation
 *--------------------*/

void dmpDataReady() {
  mpuInterrupt = true;
}

void toggleButtonInit(){
  pinMode(inPin, INPUT);
  pinMode(outPin, OUTPUT);
  state = LOW;
}

void sevenSegmentInit(){
  pinMode(latchPin, OUTPUT);
  pinMode(dataPin, OUTPUT);
  pinMode(clockPin, OUTPUT);
  magnitude = 0;
}

void lcdInit(){  
  lcd.begin(16,2);
  lcd.backlight();
}

void gyroInit(){
  
  magnitudeC = 0;
  #if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
    Wire.begin();
    Wire.setClock(400000);
  #elif I2CDEV_IMPLEMENTATION == I2CDEV_BUILTIN_FASTWIRE
    Fastwire::setup(400, true);
  #endif
  
  // initialize device
  Serial.println(F("Initializing I2C devices..."));
  mpu.initialize();
  pinMode(INTERRUPT_PIN, INPUT);
  
  // verify connection
  Serial.println(F("Testing device connections..."));
  Serial.println(mpu.testConnection() ? F("MPU6050 connection successful") : F("MPU6050 connection failed"));

  // load and configure the DMP
  Serial.println(F("Initializing DMP..."));
  devStatus = mpu.dmpInitialize();

  mpu.setXGyroOffset(220);
  mpu.setYGyroOffset(76);
  mpu.setZGyroOffset(-85);
  mpu.setZAccelOffset(1788);
  if (devStatus == 0) {
    Serial.println(F("Enabling DMP..."));
    mpu.setDMPEnabled(true);

    Serial.println(F("Enabling interrupt detection (Arduino external interrupt 0)..."));
    attachInterrupt(digitalPinToInterrupt(INTERRUPT_PIN), dmpDataReady, RISING);
    mpuIntStatus = mpu.getIntStatus();

    Serial.println(F("DMP ready! Waiting for first interrupt..."));
    dmpReady = true;

    packetSize = mpu.dmpGetFIFOPacketSize();
  } else {
    // ERROR!
    Serial.print(F("DMP Initialization failed (code "));
    Serial.print(devStatus);
    Serial.println(F(")"));
  }
}

void changeShift() {
  int i;
  for(i=0;i<TIME_WAVE_STORE-1;i++){
    aaWorldPrev[i] = aaWorldPrev[i+1];
  }
  aaWorldPrev[TIME_WAVE_STORE-1] = aaWorld;
}


bool isChange(){
  int offset = 200;
  int i;
  for(i = TIME_WAVE_STORE-1; i >= 0 ; i--){
    int dx = abs(aaWorldPrev[i].x - aaWorld.x);
    int dy = abs(aaWorldPrev[i].y - aaWorld.y);
    int dz = abs(aaWorldPrev[i].z - aaWorld.z);
    bool isChange = (dx > offset || dy > offset || dz > offset);
    if(isChange){
      double amplitudeNow = sqrt(pow(dx,2)+pow(dy,2)+pow(dz,2))/1000;
      amplitude = ((amplitudeNow>amplitude || amplitude==0.0)?amplitudeNow:amplitude);
      return true;
    }
  }
  return false;
}

double magnitudeConvert() {
  // assume the epicentral distance is 0.5km
  return ((amplitude>9)?9:amplitude);
}

void displaySevenSegment(int val){
    if(val!=valGlobal){
      digitalWrite(latchPin, LOW);
      shiftOut(dataPin, clockPin, MSBFIRST, digit[val]);
      digitalWrite(latchPin, HIGH);
      valGlobal=val;
    }
}

void ledActive(){
//  Serial.print("LED : ");
//  Serial.println(magnitudeC);
  int fadeValue = floor(((float)magnitudeC/10)*255);
//  Serial.print("Fade Value : "); Serial.println(fadeValue);
  analogWrite(ledPin, fadeValue);
}

void ledDeactive(){
  analogWrite(ledPin, 0);
}

void gyroOn(){
  if (!dmpReady) return;
  while (!mpuInterrupt && fifoCount < packetSize);
  
  mpuInterrupt = false;
  mpuIntStatus = mpu.getIntStatus();
  fifoCount = mpu.getFIFOCount();
  if ((mpuIntStatus & 0x10) || fifoCount == 1024) {
    mpu.resetFIFO();
    Serial.println(F("FIFO overflow!"));
  } else if (mpuIntStatus & 0x02) {
    while (fifoCount < packetSize) fifoCount = mpu.getFIFOCount();

    mpu.getFIFOBytes(fifoBuffer, packetSize);
    fifoCount -= packetSize;

    mpu.dmpGetQuaternion(&q, fifoBuffer);
    mpu.dmpGetAccel(&aa, fifoBuffer);
    mpu.dmpGetGravity(&gravity, &q);
    mpu.dmpGetLinearAccel(&aaReal, &aa, &gravity);
    mpu.dmpGetLinearAccelInWorld(&aaWorld, &aaReal, &q);

    // blink LED to indicate activity
    changeShift();
    bool check = isChange();
    if(check){
      iter+=1;
    }
    if(check && !detected && iter==100){
      magnitudeC = (int) magnitudeConvert();
      Serial.println(magnitudeC);
      displaySevenSegment(magnitudeC);
      ledActive();
      detected = 1;
    } else if(iter==0){
      detected = 0;
      ledDeactive();
      amplitude = 0;
      displaySevenSegment(0);
    }

    if(detected){
      iter+=1;
      iter=iter%1000;
    }
//    Serial.println(iter);
  }
}

void gyroOff(){
  displaySevenSegment(0);
  ledDeactive();
}


void printLCD(int x,int y,String str){
  lcd.setCursor(x,y);
  lcd.print(str);
}

void clearLCD(){
  lcd.clear();
}

bool isState(int state){
  int temp = stateLED;
  stateLED = state;
  return temp==state;
}

